require "droplet_kit"

module DigitalOceanInventory
  class Collector
    attr_reader :config, :client

    def self.build(config)
      new config
    end

    def initialize(config)
      @config = config
      @client = config.client
    end

    def call(inventory)
      droplets = get_droplets
                    .select { |d| select_droplet? d }
                    .reject { |d| ignore_droplet? d }

      droplets.each do |droplet|
        groups = droplet_groups droplet
        groups.each { |g| inventory.add_group g }

        inventory.add_host droplet, groups
      end

      inventory
    end

    private

      def project
        if !config.project || config.project == "default"
          client.projects.find_default
        else
          proj = client.projects.all.find { |p| p.name == config.project }
          raise "Couldn't find project with name #{config.project}" unless proj

          proj
        end
      end

      def get_droplets
        resources = client.projects.list_resources id: project.id
        resources.select { |r| r.urn.include? "do:droplet" }.map do |r|
          ns, type, id = r.urn.split ":"
          client.droplets.find id: id
        end
      end

      def select_droplet?(droplet)
        tags = droplet.tags || []
        tags.any? { |t| t == "ansible:enable" }
      end

      def ignore_droplet?(droplet)
        ignore_tags  = config.ignore_tags
        ignore_hosts = config.ignore_hosts

        if ignore_tags.empty? && ignore_hosts.empty?
          return false
        end

        return true if ignore_hosts.include? droplet.name

        tags = droplet.tags || []
        ignore_tags.any? { |t| tags.include? t }
      end

      def droplet_groups(droplet)
        groups = %w[digitalocean]
        groups << droplet.region.slug if config.group_by_region
        groups << droplet_vpc(droplet)

        droplet.tags
               &.uniq
               &.select { |t| t.match? /^ansible:g:/ }
               &.map { |t| t.gsub "ansible:g:", "" }
               &.each { |t| groups << t }

        groups.uniq.map { |g| g.gsub('-', '_') }
      end

      def droplet_vpc(droplet)
        id = droplet.vpc_uuid
        vpc = config.client.vpcs.find id: id
        vpc.name
      end
  end
end
