require "ostruct"
require_relative "./host"
require_relative "./group"

module DigitalOceanInventory
  class Inventory
    attr_reader :config, :groups, :hosts

    def self.build(config)
      all = Group.build name: "all", config: config

      groups = OpenStruct.new all: all
      hosts  = OpenStruct.new

      new config, groups, hosts
    end

    def initialize(config, groups, hosts)
      @config = config
      @groups = groups
      @hosts  = hosts
    end

    def add_group(name)
      return groups[name] if groups[name]

      group = Group.build name: name, config: config
      @groups[name] = group

      group
    end

    def add_host(droplet, host_groups = [])
      name = droplet.name

      if hosts[name]
        host = hosts[name]
      else
        host = Host.build droplet: droplet, config: config
        hosts[name] = host
      end

      groups.all << host

      host_groups.each do |g|
        if groups[g]
          groups[g] << host
        else
          group = add_group g
          group << host
        end
      end

      host
    end
  end
end
