require "json"

module DigitalOceanInventory
  class Serializer
    attr_reader :config

    def self.build(config)
      new config
    end

    def initialize(config)
      @config = config
    end

    def call(inventory)
      meta = { _meta: { hostvars: hostvars(inventory) } }
      data = groups(inventory).merge meta

      JSON.dump data
    end

    private

      def groups(inventory)
        hash = {}
        inventory.groups.to_h.each do |name, group|
          hash[name] = {
            vars: groupvars(group),
            hosts: group.hosts.map { |h| h.droplet.name }.uniq,
            children: []
          }
        end

        hash
      end

      def groupvars(group)
        return {} unless group.name == "all"

        hosts = group.hosts.map { |h| h.vars }

        {
          public_ips: hosts.map { |v| v[:public_ip] }.uniq,
          private_ips: hosts.map { |v| v[:private_ip] }.uniq,
          vpc_subnets: hosts.map { |v| v[:vpc_subnet] }.uniq
        }
      end

      def hostvars(inventory)
        inventory.hosts.to_h.values.map { |h| [h.droplet.name, h.vars] }.to_h
      end
  end
end
